/*------------------------------------------------------------------------------------
   Description: Additional Matrix for the Genesis Trading System
                - Does not repaint
                - MTF
   Based on Genesis Matrix	
   Change log:
   1.20  Changed indicator from ASCTrend to Xaphod's ASCTrend1i; 500x faster
   1.10  Added Global Variable output
         Modified the NextHigherTF to return PERIOD_M30 for PERIOD_M15, not H1.
   1.00  2012 09 09 Converted to just arrow components
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright  2012, Cody_R"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 C'0,90,255'
#property indicator_color2 Red
#property indicator_color3 C'0,90,255'
#property indicator_color4 Red
#property indicator_minimum 0
#property indicator_maximum 1.2

//#include <xPrint.mqh>

// Win32 API
#import "user32.dll"
   bool     GetWindowRect(int hWnd, int& lpRect[]);
#import
#import "Kernel32.dll"
  int CreateFileW(string lpFileName,int dwDesiredAccess, int dwShareMode, int lpSecurityAttributes, int dwCreationDisposition,int dwFlagsAndAttributes, int hTemplateFile);
  int CloseHandle(int hObject);
#import

#define OPEN_EXISTING 3
#define FILE_SHARE_READ 1
#define GENERIC_READ 0x80000000
#define FILE_ATTRIBUTE_NORMAL 128
#define INVALID_HANDLE_VALUE 0xFFFFFFFF

// Constant definitions
#define INDICATOR_NAME "GenesisArrows"
#define INDICATOR_VERSION "1.20_2"
#define BAR_CHAR 110


#define IND_Row1      "ASCTrend1i"
#define IND_Row2      "ASCTrend1i"

#define IDX_Row1      0
#define IDX_Row2      1
//#define IDX_Row3      2
//#define IDX_Row4      3

// Indicator parameters
extern string    Indi_Version=INDICATOR_VERSION;
extern string    TimeFrame_Settings="";
extern int       TimeFrame_Period=0;        // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0.
extern int       TimeFrame_Auto=0;          // Automatically select higher TF. M15 and M30 -> H1. Off=0, 1st HTF=1, 2nd HTF=2
extern string    ARR_Settings="";
extern string    ARR_Label="Arr_";
extern string    HTF_ARR_Settings="";
extern int       HTF_ARR_Period=0;          // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0. 
extern int       HTF_ARR_Auto=1;            // Automatically select higher TF. M15 and M30 -> H1. Off=0, 1st HTF=1, 2nd HTF=2
extern string    HTF_ARR_Label="Arr_";
extern string    Alert_Settings="";
extern bool      Alert_OnBarClose=True;     // Alert only when an open bar closes
extern bool      Alert_Popup=False;         // Enable popup window & sound on alert
extern string    Alert_Sound="";            // Play sound on alert. Wav files only
extern bool      Alert_Email=False;         // Enable send email on alert
extern string    Alert_Subject="";          // Email Subject. Null string ("") will result in a preconfigured subject.
extern string    Display_Settings="";
extern color     Display_LabelColor=White;  // Color of Histogram Id labels

// Global module varables
static int       crGreen  = 1;
static int       crBlue   = 1;
static int       crLong   = 1;
static int       crRed    = -1;
static int       crShort  = -1;

// Histogram
double gadRow1Up[];
double gadRow1Dn[];
double gadRow2Up[];
double gadRow2Dn[];

double RowGap[2] = {0.9, 0.4};

// Labels
double gadGap[2];
string Row1_Label,
       Row2_Label,
       Row3_Label,
       Row4_Label;

// Globals
int    giRepaintBars;
string gsIndicatorName;
bool   gbInit;
int    NumberOfRows;

//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
  int i;
  
  NumberOfRows = indicator_buffers/2;

  // Init indicator buffers
  IndicatorBuffers(indicator_buffers);
  
  SetIndexStyle(0,DRAW_ARROW);
  SetIndexBuffer(0,gadRow1Up);
  SetIndexStyle(1,DRAW_ARROW);
  SetIndexBuffer(1,gadRow1Dn);

  SetIndexStyle(2,DRAW_ARROW);
  SetIndexBuffer(2,gadRow2Up);
  SetIndexStyle(3,DRAW_ARROW);
  SetIndexBuffer(3,gadRow2Dn);

  for ( i = 0; i < indicator_buffers; i++) {
    SetIndexLabel(i,NULL);
    SetIndexEmptyValue(i,0.0);
    SetIndexArrow(i,BAR_CHAR);
  }

  // Set HTF Arr TimeFrame
    if ( HTF_ARR_Period == 0 ) HTF_ARR_Period = Period();
    
    switch(HTF_ARR_Auto) {
    case 1:
      HTF_ARR_Period=NextHigherTF(HTF_ARR_Period);
    break;
    case 2:
      HTF_ARR_Period=NextHigherTF(NextHigherTF(HTF_ARR_Period));
    break;
    default:
      if (HTF_ARR_Period<1 || HTF_ARR_Period==Period()) {
        HTF_ARR_Period=Period();
      }
    break;
  }

   HTF_ARR_Label = HTF_ARR_Label+  TF2Str(HTF_ARR_Period);
  
  // Set Timeframe
  switch(TimeFrame_Auto) {
    case 1:
      TimeFrame_Period=NextHigherTF(TimeFrame_Period);
      giRepaintBars=TimeFrame_Period/Period()+2;
    break;
    case 2:
      TimeFrame_Period=NextHigherTF(NextHigherTF(TimeFrame_Period));
      giRepaintBars=TimeFrame_Period/Period()+2;
    break;
    default:
      if (TimeFrame_Period<1 || TimeFrame_Period==Period()) {
        TimeFrame_Period=Period();
        giRepaintBars=0;
      }
      else {
        giRepaintBars=TimeFrame_Period/Period()+2;
      }
    break;
  }

  ARR_Label = ARR_Label + TF2Str(TimeFrame_Period);

  // Set histogram positions
  for (i=0; i<=NumberOfRows; i++)
  {
    gadGap[i]    = RowGap[i]; 
  }                            

  // Check that required indicators are installed
  if (!IndicatorExists(IND_Row1+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_Row1+".ex4 cannot be found.");
  if (!IndicatorExists(IND_Row2+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_Row2+".ex4 cannot be found.");

  // Misc
  gsIndicatorName=INDICATOR_NAME+" "+INDICATOR_VERSION+"("+TF2Str(TimeFrame_Period)+"."+TF2Str(HTF_ARR_Period)+")";
  IndicatorShortName(gsIndicatorName);
  gbInit=True;
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
  // Clear text objects
  for(int i=ObjectsTotal()-1; i>-1; i--)
    if (StringFind(ObjectName(i),gsIndicatorName)>=0)  ObjectDelete(ObjectName(i));
  // Clear Global Variables
  GlobalVariableDel(StringConcatenate(Symbol(), " ", INDICATOR_NAME," ", INDICATOR_VERSION, " ", TF2Str(Period() ), " ",Row1_Label ) );
  GlobalVariableDel(StringConcatenate(Symbol(), " ", INDICATOR_NAME," ", INDICATOR_VERSION, " ", TF2Str(Period() ), " ",Row2_Label ) );
  return (0);
}

//-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars;
  static datetime tCurBar;

  // Get unprocessed bars
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1);
  if(iCountedBars>0) iCountedBars--;

  // Set bars to redraw
  if (NewBars(TimeFrame_Period)>3)
    iNewBars=Bars-1;
  else
    iNewBars=Bars-iCountedBars;
  if (iNewBars<giRepaintBars)
    iNewBars=giRepaintBars;

  Row1_Label = ARR_Label;
  Row2_Label = HTF_ARR_Label;

  // Calc indicator data and update matrix
  ProcessRow1(iNewBars, gadRow1Up, gadRow1Dn, IDX_Row1);
  ProcessRow2(iNewBars, gadRow2Up, gadRow2Dn, IDX_Row2);

  // Alerts
  CheckAlert();
   
  // Tasks to execute on bar close
  if (tCurBar<Time[0]) {
    tCurBar=Time[0];

    // Write/Update bar labels
    Writelabel(ARR_Label,gadGap[IDX_Row1]   + 0.3);
    Writelabel(HTF_ARR_Label,gadGap[IDX_Row2] + 0.3);

    // Clear the init flag
    if (gbInit)
      gbInit=False;
  }

  return(0);
}


//-----------------------------------------------------------------------------
// function: NewBars()
// Description: Return nr of new bars on a TF
//-----------------------------------------------------------------------------
int NewBars(int iPeriod) {
  static int iPrevSize=0;
  int iNewSize;
  int iNewBars;
  datetime tTimeArray[];

  ArrayCopySeries(tTimeArray,MODE_TIME,Symbol(),iPeriod);
  iNewSize=ArraySize(tTimeArray);
  iNewBars=iNewSize-iPrevSize;
  iPrevSize=iNewSize;
  return(iNewBars);
}

int ProcessRow1(int iNewBars, double& vdUp[], double& vdDn[], int iRow)
{
  int i,j;
  double dSig;
  int iHTFBar=-1;

  for(i=iNewBars; i>=0; i--)
  {
    // Shift index for higher time-frame
    if (TimeFrame_Period>Period() )
      j=iBarShift(Symbol(), TimeFrame_Period, Time[i]);
    else
      j=i;
    // Calc HA
    if (iHTFBar!=j)
    {
      iHTFBar=j;
      dSig=iCustom(NULL,TimeFrame_Period,IND_Row1, 2, j);
    }
    // No Data
    if (iClose(NULL,TimeFrame_Period,j)==0) {
      vdUp[i]=EMPTY_VALUE;
      vdDn[i]=EMPTY_VALUE;
    }
    // Bull signal
    else if (dSig>=0) {
      GlobalVariableSet(StringConcatenate(Symbol(), " ", INDICATOR_NAME," ", INDICATOR_VERSION, " ", TF2Str(Period() ), " ",Row1_Label ), crLong);
      vdUp[i]=gadGap[iRow];
      vdDn[i] = 0;
    }
    // Bear signal
    else {
      GlobalVariableSet(StringConcatenate(Symbol(), " ", INDICATOR_NAME," ", INDICATOR_VERSION, " ", TF2Str(Period() ), " ",Row1_Label ), crShort);
      vdDn[i]=gadGap[iRow];
      vdUp[i] = 0;
    } 
  }
  return(0);
}

int ProcessRow2(int iNewBars, double& vdUp[], double& vdDn[], int iRow)
{
  int i,j;
  double dSig;
  int iHTFBar=-1;

  for(i=iNewBars; i>=0; i--)
  {
    // Shift index for higher time-frame
    if (HTF_ARR_Period>Period() )
      j=iBarShift(Symbol(), HTF_ARR_Period, Time[i]);
    else
      j=i;
    // Calc HA
    if (iHTFBar!=j)
    {
      iHTFBar=j;
      dSig=iCustom(NULL,HTF_ARR_Period,IND_Row2, 2, j);
    }
    // No Data
    if (iClose(NULL,HTF_ARR_Period,j)==0) {
      vdUp[i]=EMPTY_VALUE;
      vdDn[i]=EMPTY_VALUE;
    }
    // Bull signal
    else if (dSig>=0) 
    {
      GlobalVariableSet(StringConcatenate(Symbol(), " ", INDICATOR_NAME," ", INDICATOR_VERSION, " ", TF2Str(Period() ), " ",Row2_Label ), crLong);
      vdUp[i]=gadGap[iRow];
      vdDn[i] = 0;
    }
    // Bear signal
    else 
    {
      GlobalVariableSet(StringConcatenate(Symbol(), " ", INDICATOR_NAME," ", INDICATOR_VERSION, " ", TF2Str(Period() ), " ",Row2_Label ), crShort);
      vdDn[i]=gadGap[iRow];
      vdUp[i] = 0;
    } 
  }
  return(0);
}

//-----------------------------------------------------------------------------
// function: Writelabel()
// Description: Write a label for a bar
//-----------------------------------------------------------------------------
int Writelabel(string sLabel,double dPrice) {
  string sObjId;
  sObjId=gsIndicatorName+"_"+sLabel;
  if(ObjectFind(sObjId) < 0)
    ObjectCreate(sObjId, OBJ_TEXT, WindowFind(gsIndicatorName), Time[0]+Period()*60*3.5, dPrice);
  ObjectSetText(sObjId, sLabel, 8, "Calibri", Display_LabelColor); //Lucida Console
  ObjectMove(sObjId,0,Time[0]+Period()*60*3.5, dPrice);
  return(0);
}


//-----------------------------------------------------------------------------
// function: IndicatorExists()
// Description: Check if an indicator exists
//-----------------------------------------------------------------------------
bool IndicatorExists(string sIndicatorName) {
  int hFile;
  string sFile;

  // Exit if dlls are disabled
  if (!IsDllsAllowed())
    return(True);

  // Try to open indicator
  sFile=TerminalPath()+"\\MQL4\\indicators\\"+sIndicatorName;
  hFile=CreateFileW(sFile,0,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
  if (hFile==INVALID_HANDLE_VALUE) {
    return(False);
  }
  else {
    CloseHandle(hFile);
    return(True);
  }
  return(False);
}

//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1..");
    case PERIOD_M5: return("M5..");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1.");
    case PERIOD_H4: return("H4.");
    case PERIOD_D1: return("D1.");
    case PERIOD_W1: return("W1.");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
  return(0);
}


//-----------------------------------------------------------------------------
// function: NextHigherTF()
// Description: Select the next higher time-frame.
//              Note: M15 and M30 both select H1 as next higher TF.
//-----------------------------------------------------------------------------
int NextHigherTF(int iPeriod) {
  if (iPeriod==0) iPeriod=Period();
  switch(iPeriod) {
    case PERIOD_M1: return(PERIOD_M5);
    case PERIOD_M5: return(PERIOD_M15); 
    case PERIOD_M15: return(PERIOD_M30);
    case PERIOD_M30: return(PERIOD_H1);
    case PERIOD_H1: return(PERIOD_H4);
    case PERIOD_H4: return(PERIOD_D1);
    case PERIOD_D1: return(PERIOD_W1);
    case PERIOD_W1: return(PERIOD_MN1);
    case PERIOD_MN1: return(PERIOD_MN1);
    default: return(Period());
  }
}

//-----------------------------------------------------------------------------
// function: CheckAlert()
// Description: Check for new alerts
//-----------------------------------------------------------------------------
void CheckAlert() {
  static datetime tAlertBar;
  static int iPrevAlert=0;
  
  if (Alert_Popup || Alert_Email || Alert_Sound!="") {
    
    // Alert on the close of the current bar
    if (Alert_OnBarClose && tAlertBar<Time[iBarShift(Symbol(), TimeFrame_Period, Time[0])] && !gbInit) {
      
      tAlertBar=Time[iBarShift(Symbol(), TimeFrame_Period, Time[0])];
      // Clear alert flag
      if (iPrevAlert==1 && !UpSignal(1))
         iPrevAlert=0;
      else if (iPrevAlert==-1 && !DnSignal(1))
         iPrevAlert=0;
      // Alert and set alert flag
      if (UpSignal(1) && iPrevAlert!=1) {
         AlertNow(Symbol()+", "+TF2Str(TimeFrame_Period)+": Genesis Arrows Buy Signal.");
         iPrevAlert=1;
      }  
      else if (DnSignal(1) && iPrevAlert!=-1) {
         AlertNow(Symbol()+", "+TF2Str(TimeFrame_Period)+": Genesis Arrows Sell Signal.");
         iPrevAlert=-1;
      }
    }
  
    // Alert while the current bar is open
    if (!Alert_OnBarClose && tAlertBar<Time[iBarShift(Symbol(), TimeFrame_Period, Time[0])] && !gbInit) {
      // Clear alert flag
      if (iPrevAlert==1 && !UpSignal(0))
         iPrevAlert=0;
      else if (iPrevAlert==-1 && !DnSignal(0))
         iPrevAlert=0;      
      // Alert and set alert flag
      if (UpSignal(0) && iPrevAlert!=1) {
         AlertNow(Symbol()+", "+TF2Str(TimeFrame_Period)+": Genesis Arrows Buy Signal.");
         iPrevAlert=1;
         tAlertBar=Time[iBarShift(Symbol(), TimeFrame_Period, Time[0])];
      }  
      else if (DnSignal(0) && iPrevAlert!=-1) {
         AlertNow(Symbol()+", "+TF2Str(TimeFrame_Period)+": Genesis Arrows Sell Signal.");
         iPrevAlert=-1;
         tAlertBar=Time[iBarShift(Symbol(), TimeFrame_Period, Time[0])];
      }
    }
  }
  return;
}

//-----------------------------------------------------------------------------
// function: AlertNow()
// Description: Signal the popup and email alerts
//-----------------------------------------------------------------------------
void AlertNow(string sAlertMsg) {
  //Popup Alert 
  if (Alert_Popup) 
    Alert(INDICATOR_NAME, ", ", sAlertMsg);
  if (Alert_Sound!="")
    PlaySound(Alert_Sound);
  //Email Alert
  if (Alert_Email) {
    if (Alert_Subject=="")
      SendMail( sAlertMsg, "MT4 Alert!\n"+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
    else 
      SendMail( Alert_Subject, "MT4 Alert!\n"+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);          
  }
  
  return;
}

//-----------------------------------------------------------------------------
// function: UpSignal()
// Description: Return true if there is an up signal
//-----------------------------------------------------------------------------
bool UpSignal(int i) {
  if (gadRow1Up[i]>0 && gadRow2Up[i]>0 )//&& gadR3Up[i]>0 && gadR4Up[i]>0)
    return(True);
  else
    return(False);
}


//-----------------------------------------------------------------------------
// function: DnSignal()
// Description: eturn true if there is a down signal
//-----------------------------------------------------------------------------
bool DnSignal(int i) {
  if (gadRow1Dn[i]>0 && gadRow2Dn[i]>0 )//&& gadR3Dn[i]>0 && gadR4Dn[i]>0)
    return(True);
  else
    return(False);
}

